<?php

namespace Ig\IgAccountCreate\Controller;

use Ig\IgAccountCreate\Domain\Repository\FrontendUserRepository;
use Psr\Http\Message\ResponseInterface;
use Symfony\Component\Mime\Address;
use TYPO3\CMS\Core\Mail\FluidEmail;
use TYPO3\CMS\Core\Mail\Mailer;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Http\ForwardResponse;

/**
 * AccountController
 */
class AccountController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    //private $secret = 'G%rT4*D7EjdhKDFw-.,Fpo)d0';
    /**
     * FrontendUserRepository
     *
     * @var \Ig\IgAccountCreate\Domain\Repository\FrontendUserRepository
     */
    protected $frontendUserRepository = null;

    protected $utility = null;
    protected $piVars = [];
    protected $sendcode = '';
    protected $sendcode_old = '';
    protected $errors = 0;
    protected $error_email = 0;
    protected $error_password = 0;
    protected $error_password_text = 0;
    protected $error_user_exists = 0;
    protected $error_agb = 0;
    protected $minPasswordLength = 6;
    
    public function __construct(
        FrontendUserRepository $frontendUserRepository
    ) {
        $this->frontendUserRepository = $frontendUserRepository;
    }


    public function initializeAction(): void
    {
        $this->utility = GeneralUtility::makeInstance(\Ig\IgAccountCreate\Lib\Utility::class);
        $vars = array_merge($_GET, $_POST);
        $this->piVars = $vars['tx_igaccountcreate_pi1'] ?? [];
        //$this->request->getArguments()


        $t = round(strftime('%s') / 3600);
      
        $this->sendcode = md5($t . ($this->settings['secret']));
        $this->sendcode_old = md5(($t - 1) . $this->settings['secret']);
        $this->errors = 0;


        $this->utility->init($this->piVars, $this->settings, $this->sendcode);
    }

    /**
     * action register
     */
    public function registerAction(): ResponseInterface
    {
        $hash = $_REQUEST['h'] ?? '';
        $sendcode = $this->piVars['sendcode'] ?? '';
        // Token gueltig?
        if (strlen($hash) > 10 || ($sendcode == $this->sendcode || $sendcode == $this->sendcode_old)) {
            $send = 1;
        } else {
            $send = 0;
        }


        if ($send) {
            $this->error_email = 0;
            $this->error_password = 0;
            $this->error_user_exists = 0;

            $email = $this->piVars['e'] ?? '';
            $password = $this->piVars['p'] ?? '';
            $password2 = $this->piVars['p2'] ?? '';

      
            // Ist E-Mail gueltig
            $emailAscii = idn_to_ascii($email);
            if (!filter_var($emailAscii, FILTER_VALIDATE_EMAIL)) {
                $this->error_email = 1;
            } else {
                $row = $this->frontendUserRepository->findOneByEmail($email);
                if (!empty($row)) {
                    $this->error_user_exists = 1;
                }
            }

            if ($password !== $password2) {
                $this->error_password_text = 'Die beiden Passwörter sind nicht gleich';
                $this->error_password = 1;
            } elseif (strlen($password) < $this->minPasswordLength) {
                $this->error_password = 1;
                $this->error_password_text = 'Das Passwort muss mindestens ' . $this->minPasswordLength . ' Zeichen lang sein';
            }
            $this->errors = $this->error_email || $this->error_password || $this->error_user_exists;
        }
    
        if (!$this->errors && $send == 1) {
            return new ForwardResponse('registersend');
        }
        return $this->htmlResponse(
            $this->utility->get_register_form_1(
                $this->errors,
                $this->error_password_text,
                $this->error_password,
                $this->error_email,
                $this->error_user_exists
            )
        );
    }
    /**
     * action registersend (not defined in plugin only used above)
     */
    public function registersendAction(): ResponseInterface
    {
        $email = $this->piVars['e'] ?? '';
        $password = $this->piVars['p'] ?? '';
        $newsletter = (int) ($this->piVars['newsletter'] ?? false);


        
        $hash = md5($this->settings['secret'] . $email . strval(time()));

        // Create User
        $this->frontendUserRepository->createuser(
            $email,
            $password,
            $hash,
            $newsletter,
            $this->settings['thanks']['settings_pidUsers'],
            $this->settings['thanks']['settings_usergroup']
        );
        // Send Mail
        $this->sendmail($email, $password, $hash);

        return $this->htmlResponse();
    }

    
    /**
     * action confirm
     */
    public function confirmAction(): ResponseInterface
    {
        $hash = $_REQUEST['h'] ?? '';
        $sendcode = $this->piVars['sendcode'] ?? '';
        // Token gueltig?
        if (strlen($hash) > 10 || ($sendcode == $this->sendcode || $sendcode == $this->sendcode_old)) {
            $send = 1;
        } else {
            $send = 0;
        }


        // Formular 2
        // Felder testen
        if ($send) {
            $this->error_agb = 0;
        }
        
        $this->errors = $this->errors || $this->error_agb;
        if (!$this->errors && $send == 1) {
            //return 'confirm';//$content = $this->update_register_form_2();
        } else {
            return new ForwardResponse('data');
        }
        $affectedRows = $this->frontendUserRepository->enableUser($hash);
        $this->view->assign('hasError', $affectedRows != 1);

        return $this->htmlResponse();
    }

    /**
     * action data
     */
    public function dataAction(): ResponseInterface
    {
        $hash = $_REQUEST['h'] ?? '';
        //echo('h='.$hash);
        if (strlen($hash) < 10) {
            return $this->htmlResponse('Ungültiger Link');
        }

        
        $row = $this->frontendUserRepository->findOneByComments($comments);
        if (!$row) {
            return $this->htmlResponse('Ungültiger Link');
        }

        
        $content = $this->utility->get_register_form_2();
        return $this->htmlResponse($content);
    }


    public function sendmail($email, $password, $hash/*, $valid_to*/)
    {
        $baseUrl = GeneralUtility::getIndpEnv('TYPO3_SITE_URL');
        //$endtime= date("d.m.Y - H:i", $valid_to);
        $msg = $this->settings['thanks']['email_text'];
        
        $msg = str_replace('###USERNAME###', $email, $msg);
        $msg = str_replace('###PASSWORD###', $password, $msg);

        $username_password = '<table><tr><td style="font-size:10pt;font-family:arial;padding-left: 0;">Benutzername:</td><td>&nbsp;</td><td style="font-size:10pt;font-family:arial;;padding-left: 0;">' . htmlspecialchars(
            $email
        ) . '</td></tr>
<tr><td style="font-size:10pt;font-family:arial;;padding-left: 0;">Passwort:</td><td>&nbsp;</td><td style="font-size:10pt;font-family:arial;;padding-left: 0;">' . htmlspecialchars(
    $password
) . '</td></tr></table>';
        $msg = str_replace('###USERNAME_PASSWORD###', $username_password, $msg);

        $msg = str_replace(
            '###LOGIN_LINK###',
            '<a href="' . $baseUrl . 'de/login.html">' . $baseUrl . 'de/login.html</a>',
            $msg
        );
        $msg = str_replace(
            '###CONFIRM_LINK###',
            '<a href="' . $baseUrl . 'de/registrieren/bestaetigungsseite.html?h=' . urlencode(
                $hash
            ) . '">' . $baseUrl . 'de/registrieren/bestaetigungsseite.html?tx_igaccountcreate_pi1[send]=1&h=' . urlencode(
                $hash
            ) . '</a>',
            $msg
        );
        $msg = str_replace(
            '###EMAIL_LINK###',
            '<a href="mailto:' . $this->settings['thanks']['email_from'] . '">' . $this->settings['thanks']['email_from'] . '</a>',
            $msg
        );



        // Mail Verschicken
        //$cObj = GeneralUtility::makeInstance ( "TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer" );
        //$parseFunc = $GLOBALS ['TSFE']->tmpl->setup ['lib.'] ['parseFunc_RTE.'];
        $contact = ['Nähcenter Iseli AG', 'Oberlandstrasse 17', '3700 Spiez', 'Telefon +41 33 654 34 84'];
        
        $mail = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Mail\MailMessage::class);
        $parameter = [
            'msg' => $msg, //$cObj->parseFunc ( $msg, $parseFunc ),
            'logo' => 'fileadmin/layout/images_layout/naehcenter-iseli.jpg',
            'contact' => $contact,
            'baseUrl' => $baseUrl,
        ];
        /*var_dump($parameter);
          die();*/

        $fluidEmail = GeneralUtility::makeInstance(FluidEmail::class);
        $fluidEmail
            ->setRequest($this->request)
            ->to($email)
            ->from(new Address($this->settings['thanks']['email_from']))
            ->subject($this->settings['thanks']['email_subject'])
            ->format(FluidEmail::FORMAT_HTML) // send HTML and plaintext mail
            ->setTemplate('IgAccountCreate/Index')
            ->assignMultiple($parameter);

        GeneralUtility::makeInstance(Mailer::class)->send($fluidEmail);
        
        /*$body = $this->renderFluid("Mail/Index.html", $parameter);
          $mail->setFrom(array($this->settings['thanks']['email_from']));
          $mail->setTo(array($email));
          $mail->setSubject($this->settings['thanks']['email_subject']);
          $mail->send();*/
    }
    /**
     * renders fluid template
     * @param array $parameter key=>value pairs, where key is fluid placeholder {key} in the template
     * @return string rendered (html-format) template
     */
    private function renderFluid($templateFile, $parameter)
    {
        $fluidView = GeneralUtility::makeInstance(\TYPO3\CMS\Fluid\View\StandaloneView::class);
        $extensionName = $this->request->getControllerExtensionName();
        //$fluidView->getRequest()->setControllerExtensionName($extensionName);
        $extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK
        );
        $fluidView->setLayoutRootPaths($extbaseFrameworkConfiguration['view']['layoutRootPaths']);
        $fluidView->setPartialRootPaths($extbaseFrameworkConfiguration['view']['partialRootPaths']);
        $fluidView->setTemplateRootPaths($extbaseFrameworkConfiguration['view']['templateRootPaths']);
        $fluidView->setTemplate($templateFile);
 
        $fluidView->assignMultiple($parameter);
        return $fluidView->render();
    }
}
